'''
WalArt
=============================
Walnut Artifacts

Created by Cerulany 2015 <www.zhihu.com/people/cerulany>

Making scientific exploration more friendly

'''

'''
Revisions for alib.py:
==============================
151102 alib
151104 alib.Load,Save
151114 add auto indexing to *alib.FromString()*
151125 removed alib.type because __class__.__mro__ is available
151210 add *alib.Parse*,*setValue*,*getValue*
151218 add *alib.Merge*
151219 add *alib.Append* to use it as a list, *Pop*, *FindLastIndex*
         modified alib.ToString() to include object values
         *ToList*
160114 add *Call*, this will simplify the execution process
160204 modified alib.setValue, alib.getValue for custom separator
160602 Cancelled extension appending in alib.Save and .Load for clarity
161107 Added exception handling in alib.Call
161117 add filename display in alib.Call
161126 modified alib.ToString for more clear presentation
180326 fixed alib.ToString to eliminate recursion
180327 fixed alib object not hashable, using object.__repr__(alib)
180424 improved getValue for path check
    add string input support in __init__
    upgraded alib.ToList to support name assignment
    override alib.__getattr__ for more convenient reference, save __setattr__ when necessary
180604 fixed alib.FromString when the first character is \r
181104 add if isinstance(self[i],list) or isinstance(self[i],tuple) to fix the ambiguous problem
190828 repaired alib.__getattr__ to make alib support pickling and parallel processing
190905 moved alib and alibiter out of WalArt main package init code,
    to make it more accessible and graftable
190920 repaired head import for a weird error
200524 [source code]added kwargs to alib init
200531 [source code]added __hash__ to alib, return self.ToString()
220818 modify to be compatible with micropython
    line 143 object.__repr__ to repr()
    self[k]=content... to self.__setitem__(k,content...) micropython crash (core 1 panic) but Computer OK
    add DEBUG
'''
try:
    from . import waText
    from . import waFile
except ImportError:
    import waText
    import waFile

DEBUG=False

class alibiter(object):
    ''' iterator for alib: output numberical keys first, then in alphabetical order
'''
    def __init__(self,A):
        self.i=1 #next number, 0 for no number
        self.a=A
        self.list=list(A.keys())
        self.list.sort()

    def __next__(self):
        if len(self.list)==0:
            raise StopIteration
        elif self.i>0 and str(self.i) in self.a:
            self.list.remove(str(self.i))
            self.i+=1
            return str(self.i-1)
        else:
            self.i=0
            return self.list.pop(0)

class alib(dict):
    '''
A tree-like structure with string keys.
Used for general information recording and knowledge representation
Equivalent to AttributeLibrary in CV
The values can be str, or another alib or an object

The deflated alib will have all string representations,
while the inflated one can have object as value
'''
    def __init__(self,d={},**kwargs):
        '''
Converts a dict to corresponding alib by converting its keys to strings
'''
        try:
            dict.__init__(self)
        except AttributeError:
            pass
        self.type='alib'
        if isinstance(d,dict):
            for i in d:
                self[str(i)]=d[i]
        elif isinstance(d,str):
            self.FromString(d)
        if kwargs:
            for k in kwargs:
                self[k]=kwargs[k]

    def __iter__(self):
        '''To traverse in alphabetical order
'''
        return alibiter(self)
    def __str__(self):
        return self.ToString()

    def __getattr__(self,name):
        ''' favors intrinsic values'''
        try:
            return self.__dict__[name]
        except KeyError:
            try:
                return self[name]
            except KeyError:
                raise AttributeError
    def __hash__(self):
        return hash(self.ToString())


    def ToString(self,indent=False,visited=None):
        '''
Converts an alib to its string representation.
e.g. [a|b][c|d][e|[f|g][h][i|[j|k]]]
indent: the leading white spaces
'''
        result=''
        if indent!=False:
            ni=indent+'    '#next indent
            begin='\n' #str to add at the begining of a new expressive node
        else:
            ni=False
            begin=''
        for i in self:
            if indent!=False:
                if result:
                    result+='\n'+indent
                else:
                    result=indent
            if isinstance(self[i],alib):
                if not visited:
                    visited={}
                sh=repr(self[i]) #a unique identifier for the alib throughout its lifetime
                if sh in visited:
                    result+='['+i+'|...]'
                else:
                    visited[sh]=True
                    result+='['+i+'|'+begin+self[i].ToString(ni,visited)+']'
            else:
                result+='['+i
                if isinstance(self[i],list) or isinstance(self[i],tuple):
                    result+='|'+str(self[i])+']'
                elif self[i]==None or isinstance(self[i],str) and len(self[i])==0:
                    result+=']'
                else:
                    result+='|'+str(self[i])+']'

        return result
    def Parse(s):
        '''parse string s, if can be parsed to alib, return alib, else return s
'''
        ss=s.strip()
        if len(ss)>0 and ss[0]=='[':
            return alib().FromString(ss)
        else:
            return s


    def FromString(self,s):
        '''Convert from the string form, clear original data
'''
        try:
            self.clear()
        except TypeError:
            # microPython will get here
            pass
        content,cur=waText.SkipUntil(s,'[',0)
        lens=len(s)
        i=1
        while cur<lens:
            if s[cur]=='[':
                content,cur = waText.ReadSimpleBrackets(s,'[]',cur)
                col=content.find('|')
                if col==-1:
                    self[content]=''
                    content,cur=waText.SkipUntil(s,'[',cur)
                    #if no content marker | found, treat it just as a name, and continue to next block
                    continue
                elif col==0: #then auto index
                    while str(i) in self:
                        i+=1
                    k=str(i)
                else:
                    #in the normal case, the name is before|
                    k=content[0:col]
                #then treat the value part
                if col+1>=len(content):
                    self[k]=''
                elif content[col+1]=='[':
                    self[k]=alib().FromString(content[col+1:])
                elif content[col+1]=='\r' or content[col+1]=='\n':
                    #dealing with the pretty print trick
                    #only if the next line is whitespace and then start with [
                    wsp,ncur=waText.SkipUntil(content,'[',col+1)
                    if wsp.strip()=='':
                        #then should be treated as an alib
                        self[k]=alib().FromString(content[col+1:])
                    else:
                        self[k]=content[col+1:] #treat as a normal string
                else:
                    if DEBUG:
                        print('{} {} assigns {}'.format(type(self),repr(k),repr(content[col+1:])))
                    #self[k]=content[col+1:]
                    self.__setitem__(k,content[col+1:])
            content,cur=waText.SkipUntil(s,'[',cur)
        return self
    def Load(self,filename):
        '''Load from text file
'''
        #if waFile.GetExtension(filename)=='':
        #    filename+='.alib.txt'
        s=waFile.LoadText(filename)
        self.FromString(s)
        return self
    def Save(self,filename):
        '''Save contents to text file
'''
        #if waFile.GetExtension(filename)=='':
        #    filename+='.alib.txt'
        s=self.ToString('')
        waFile.SaveText(filename,s)
    def getValue(self,path,separator='|'):
        '''Get the value of given path
path can be a string like a|b|c or a list
if the path does not exist, raise IndexError
'''
        if isinstance(path,str):
            path=path.split(separator)
        k=path.pop(0)
        v=self[k]
        for k in path:
            if isinstance(v,dict):
                v=v[k]
            else:
                raise KeyError('Path ({}) is too long, stopped at {}'.format(path,k))
        return v
    def setValue(self,path,v,separator='|'):
        '''Set the value of given path, create if necessary
others same as getValue.
'''
        if isinstance(path,str):
            path=path.split(separator)
        d=self
        parent=None
        #last=path.pop()
        for i in range(len(path)):
            k=path[i]
            if isinstance(d,alib):
                if k in d:
                    parent=d
                    d=d[k]
                else:
                    parent=d
                    d[k]=alib()
                    d=d[k]
            else:
                #d is a string or something else
                parent[path[i-1]]=alib()
                parent=parent[path[i-1]]
                parent[k]=''
                d=''
        if parent==None:
            raise KeyError('path is empty')
        else:
            parent[path[-1]]=v
    def Merge(self,a2):
        '''Merge a2 in. Only update new keys.
Return merged self
'''
        for k2 in a2:
            if k2 not in self:
                self[k2]=a2[k2]
        return self
    def FindLastIndex(self):
        '''Returns the last index (int), if no numeric indices, return 0
'''
        i=1
        while str(i) in self:
            i+=1
        return i-1

    def Pop(self,k,d=None):
        '''Same as dict.pop, but also updates list if k is int
'''
        if isinstance(k,int):
            kp=k+1
            k=str(k)
            r=d
            if k in self:
                r=self.pop(k,d)
                while str(kp) in self:
                    self[k]=self[str(kp)]
                    k=str(kp)
                    kp+=1
                if k in self:
                    self.pop(k)
            return r
        else:
            return self.pop(k,d)
    def Append(self,data):
        '''Append data to the last index
'''
        i=self.FindLastIndex()
        self[str(i+1)]=data
    def ToList(self,names=None):
        '''Convert the numeric content to a list, start from 1
'''
        l=[]
        if not names:
            i=1
            while str(i) in self:
                l.append(self[str(i)])
                i+=1
        else:
            for n in names:
                l.append(self[n])
        return l
    def Call(self,s,args,glbs=globals()):
        '''Execute string indicated in s, using args as arguments,
and returns ans if available
'''
        if s in self:
            code=self[s]
        else:
            code=s
        try:
            exec(compile(code,filename=s,mode='exec'),glbs,locals())
        except Exception as e:
            if 'message' in e.__dict__:
                e.message+='\nIn calling %s with (%s)'%(s,args)
            raise e
        if 'ans' in locals():
            #print(locals())
            return locals()['ans']
        else:
            return None
    __call__=Call

#demo mode
if __name__ == '__main__':
    a=alib().FromString('[a|b][c|d][e|[f|g][h][i|[j|k]]]')
    a.Save('a')




#星尘协议之信使
#stat和msg变量用于控制_thread控制的异步
#wsStat和wsMsg用于控制asyncio控制的异步
#workModuel是当前运行的模块；menu菜单也是一个模块，属于基础模块
#shipei记录的是调用的那种适配类
#output函数用于同时向网页和单色屏显示内容
#由于windows没有单色屏，用print模拟，可以再控制台预览输出
class starMessager:
    def __init__(self):
        self.stat = "waiting"
        self.msg = ""
        try:
            from . import wa
        except:
            from wadapter import wa
        self.workModule = None
        self.wsStat = "waiting"
        self.wsMsg = ""
        self.shipei = wa
    def output(self,str):
        if sm.shipei.name == 'win':
            print("{}\n{}\n{}".format("-"*20,str,"-"*20))
        self.wsStat = "screen"
        self.shipei.DrawStdLines(str)
        self.wsMsg = str
sm = starMessager()

#基于线程的输入输出控制
if sm.shipei.name == 'zkb':
    from mpython import *
    def inputP(_):
        sm.workModule.InputKey("P")
    def inputY(_):
        sm.workModule.InputKey("Y")
    def inputT(_):
        sm.workModule.InputKey("T")
    def inputH(_):
        sm.workModule.InputKey("H")
    def inputO(_):
        sm.workModule.InputKey("O")
    def inputN(_):
        sm.workModule.InputKey("N")
    def inputA(_):
        sm.workModule.InputKey("A")
    def inputB(_):
        sm.workModule.InputKey("B")
    touchpad_p.event_pressed = inputP
    touchpad_y.event_pressed = inputY
    touchpad_t.event_pressed = inputT
    touchpad_h.event_pressed = inputH
    touchpad_o.event_pressed = inputO
    touchpad_n.event_pressed = inputN
    button_a.event_pressed = inputA
    button_b.event_pressed = inputB

########## ws服务器实体  ################
if sm.shipei.name != 'zkb':
    import asyncio
    import websockets
    import json

    wsPort = '8525'

    def d2j(dict):
        return json.dumps(dict,ensure_ascii=False)

    ScreenSize = (240,120)
    async def sendscreen(websocket,svgContent):
        try:
            from gTixi import SvgScreen
        except:
            from wadapter import SvgScreen
        await websocket.send(d2j({"type":"screen","msg":SvgScreen(ScreenSize).DrawStdLines(svgContent)}))

    if sm.shipei.name == 'smp':
        def hardIn():
            while True:
                inputmsg = sm.shipei.CheckKey()
                sm.workModule.InputKey(inputmsg)

        import threading
        hin = threading.Thread(target=hardIn)
        hin.start()

    async def sendKey():
        while True:
            while sm.stat == "waiting":
                import time
                await asyncio.sleep(0.05)
            if sm.stat == "key":
                sm.workModule.InputKey(sm.msg)
            sm.stat = "waiting"
            if sm.wsStat == "break":
                sm.wsStat="waiting"
                break

    async def webOut(websocket):
        while True:
            while sm.wsStat == "waiting":
                await asyncio.sleep(0.05)
            if sm.wsStat == "screen":        
                await sendscreen(websocket,sm.wsMsg)
            sm.wsStat = "waiting"
            if sm.wsStat == "break":
                sm.wsStat="waiting"
                break

    async def webIn(websocket):
        while True:
            try:
                message = await websocket.recv()
            except:
                sm.wsStat="break"
                break
            message = json.loads(message)
            if message['type'] == 'key':
                sm.stat = "key"
                sm.msg = message['msg']

    async def echo(websocket, path):
        await sendscreen(websocket,"欢迎来到星尘演示机\n")
        await asyncio.sleep(1)
        sm.workModule.show()
        await asyncio.gather(webIn(websocket),webOut(websocket),sendKey())
        
    loop = asyncio.new_event_loop()
    print('Star Socket Server Start!')
    print('---------------------------')
    asyncio.set_event_loop(loop)
    loop.run_until_complete(websockets.serve(echo, '0.0.0.0', wsPort))
    def wsStart():
        loop.run_forever()
    def wsStop():
        loop.stop()

    import threading
    wsServer = threading.Thread(target=wsStart)
    class wsCtrl():
        def __init__(self):
            self.loop = loop
            self.t1 = threading.Thread(target=wsStart)
        def stop(self):
            loop.stop()
        def start(self):
            if not self.loop.is_running():
                if self.t1.is_alive():
                    self.t1.join()
                self.t1.start()
                if sm.shipei.name == 'smp':
                    sm.shipei.InitDisplay()
else:
    wsServer = None
    class wsCtrl():
        pass

######### WEBAPP ROUTERS ###############
"""
This is a sample for qpython webapp
"""
if sm.shipei.name != 'zkb':#只有bdz需要自己建立bottle架构的网络服务器
    from bottle import Bottle, ServerAdapter
    from bottle import run, debug, route, error, static_file, template, post

    ######### QPYTHON WEB SERVER ###############

    class MyWSGIRefServer(ServerAdapter):
        server = None

        def run(self, handler):
            from wsgiref.simple_server import make_server, WSGIRequestHandler
            if self.quiet:
                class QuietHandler(WSGIRequestHandler):
                    def log_request(*args, **kw): pass
                self.options['handler_class'] = QuietHandler
            self.server = make_server(self.host, self.port, handler, **self.options)
            self.server.serve_forever()

        def stop(self):
            #sys.stderr.close()
            import threading
            threading.Thread(target=self.server.shutdown).start()
            #self.server.shutdown()
            self.server.server_close() #<--- alternative but causes bad fd exception
            print("# qpyhttpd stop")

    def httpSrv():
        ######### BUILT-IN ROUTERS ###############
        @route('/__exit', method=['GET','HEAD'])
        def __exit():
            global server
            server.stop()

        @route('/<filepath:path>')
        def serverstatic(filepath):
            if sm.shipei.name == 'bdz':
                return static_file(filepath, root='/sdcard/qpython/projects3/WalArtMinS/stardevice/')
            else:
                return static_file(filepath,root='./stardevice')


        ######### WEBAPP ROUTERS ###############
        @route('/')
        def home():
            if sm.shipei.name == 'bdz':
                return static_file("index.html", root='/sdcard/qpython/projects3/WalArtMinS/stardevice/')
            else:
                return static_file("index.html",root='./stardevice')
        @post('/deviceStart')
        def startws():
            wsCtrl().start()
        @post('/deviceStop')
        def stopws():
            wsCtrl().stop()

        app = Bottle()
        app.route('/', method='GET')(home)
        app.route('/__exit', method=['GET','HEAD'])(__exit)
        app.route('/<filepath:path>', method='GET')(serverstatic)
        app.post('/deviceStop',method='POST')(stopws)
        app.post('/deviceStart',method='POST')(startws)
        server = MyWSGIRefServer(host="0.0.0.0", port="8081")
        app.run(server=server,reloader=False)
    import threading
    httpServer = threading.Thread(target=httpSrv)
else:
    httpServer = None




#! /usr/bin/env python3

"""
waFile
=======================
Created by Cerulany 2015 <www.zhihu.com/people/cerulany>
Deals with simple file processing

"""
# NOTE: the actual command documentation is collected from docstrings of the
# commands and is appended to __doc__ after the class has been defined.
"""
Revisions
=================
151104 LoadText, SaveText
151209 *Find*,*Join*
160202 *uigetfullfile*
180211 updated .Join to support multiple paths, conform with os.path.join
180315 added uigetfulldir
180317 modifed Join to account for /
    fixed uigetfulldir
180318 fixed Join
180320 fixed Join again, 3.4.3 does not support (path,*name)
180408 use io instead of codecs for automatic \n conversion, also specify utf-8

210609 add GetFolderName None check
220330 add tkinter check
220818 canceled codecs import, 
    change ModuleNotFoundError to ImportError to fit microPython
"""
import os
#import codecs
def LoadText(filename):
        '''Load text from *filename*
    '''
        #f=codecs.open(filename, 'r')
        import io
        with io.open(filename,'r',encoding='utf-8') as f:
                s=f.read()
        return s

def SaveText(filename,s):
        '''Save string *s* to filename
'''
        #f=codecs.open(filename, 'w')
        import io
        with io.open(filename,'w',encoding='utf-8') as f:
                s=f.write(s)

def GetFilename(fullfile):
        '''Get the file name from *fullfile* name
'''
        return os.path.split(fullfile)[-1]
def GetFolderName(fullfile):
    if fullfile is not None:
        return os.path.dirname(fullfile)
def GetExtension(filename):
        '''Get the extension
'''
        f=GetFilename(filename)
        i=f.rfind('.')
        if i==-1:
                return ''
        else:
                return f[i:]
def Find(name):
        ''' Find and return the fullname within os.sys.path if the name exists
including .pth files
if not found, return None
'''
        for d in os.sys.path:
                if os.path.exists(os.path.join(d,name)):
                        return os.path.join(d,name);
                elif os.path.isdir(d):
                        #search .pth files
                        if d!='':
                                ds=os.listdir(d)
                        else:
                                ds=os.listdir()
                        for p in ds:
                                if GetExtension(p)=='.pth':
                                        d2=LoadText(os.path.join(d,p))
                                        if os.path.exists(os.path.join(d2,name)):
                                                return os.path.join(d2,name)
        return None
                                        
def Join(path,*name):
        ''' os.path.join
'''
        if '/' in path:
            names=[path]
            names.extend(name)
            return '/'.join(names)
        else:
            return os.path.join(path,*name)
            
try:
        import tkinter as tk
        from tkinter import filedialog
        def uigetfullfile(basefilename=''):
                '''Act like uigetfile in matlab, returned filename can be multiple
        '''
                d=GetFolderName(basefilename)
                root=tk.Tk()
                root.withdraw()
                if len(d)>0:
                        file=filedialog.askopenfilenames(initialdir=d)
                else:
                        file=filedialog.askopenfilenames()
                fns=root.tk.splitlist(file)
                return fns
        
        def uigetfulldir(basename=''):
                #d=GetFolderName(basefilename)
                root=tk.Tk()
                root.withdraw()
                if basename:
                        pth=filedialog.askdirectory(initialdir=basename)
                else:
                        pth=filedialog.askdirectory()
                return pth
except ImportError:
        print('importing waFile: tkinter is not available, uigetfullfile and uigetfulldir are not enabled')



#! /usr/bin/env python3

"""
WaText
=======================

Deals with simple text processing

Created by Cerulany 2015 <www.zhihu.com/people/cerulany>


"""
# NOTE: the actual command documentation is collected from docstrings of the
# commands and is appended to __doc__ after the class has been defined.
"""
Revisions
=================
151102 added List
151129 modified *Brief* add str size
        add *ReadBrackets*
151205 modified *ReadBrackets*
        add *ReadLine*
151219 extended *Brief* for variable length
        add *atoi* ,*Escape*,*Unescape*
160112 add type judge for Brief
171207 fixed error reporting in brackets not paired
"""

def List(obj):
        ''' List the contents of a list or dict
        obj: a list or dict
    '''
        result=''
        if isinstance(obj,list):
            result='List Contents:\n===================='
            for i in obj:
                result+='\n'+str(i)
        elif isinstance(obj,dict):
            result='Dict Contents:\n===================='
            for i in obj:
                result+='\n'+str(i)+' : '+str(obj[i])
        return result

def NegComp(a,b):
        '''Returns if a<b, but -1 is defined to be larger than any positive number here
'''
        if a==-1:
                return False
        elif b==-1:
                return True
        else:
                return a<b
        
def ReadSimpleBrackets(s,brackets='[]',cur=0):
        '''Read the content inside the brackets, cur should be pointing to bracket[0]
s: string containing the brackets and contents
brackets: must be single char, e.g. '()','{}'
cur: cursor position to begin with

rtype:(content,lastCursor)
content: content without the brackets
lastCursor: last cursor position at the end of bracket

returns '' and cur doesn't move if no valid brackets are found
raises ValueError if brackets in s are not paired
'''
        if s[cur]!=brackets[0]:
                return ('',cur)
        org=cur
        minl=s.find(brackets[0],cur+1)
        minr=s.find(brackets[1],cur+1)
        depth=1
        lens=len(s)
        while cur<lens:
                if NegComp(minr,minl):
                        cur=minr
                        minr=s.find(brackets[1],cur+1)
                        depth-=1
                else:
                        if minr==-1:
                                #print('in reading {%s}'%s)
                                raise ValueError('brackets not paired in {'+s[:cur]+'}')
                        cur=minl
                        minl=s.find(brackets[0],cur+1)
                        depth+=1
                if depth==0:
                        break
        return s[org+1:cur],cur+1

def SkipUntil(s,sub,cur=0):
        '''Skip contents until the appearance of the substring.
If sub not encountered, return until end.
rType:(content,subPos)
'''
        org=cur
        cur=s.find(sub,org)
        if cur==-1:
                return s[org:],len(s)
        else:
                return s[org:cur],cur
def ReadLine(s,cur=0):
        '''Return the contents of current line(without \n symbol) and move the cursor to the start of next line
If string ends, return len(s) instead of new line cur
rType: (content,cur)
'''
        r,cur=SkipUntil(s,'\n',cur)
        if s[cur]=='\n':
                cur+=1
        return r,cur
        
def Brief(s,length=30):
        '''Reture the brief of string s for no more than *length*, for quick one-line review
'''
        if not isinstance(s,str):
                return s
        
        lens=len(s)
        
        if lens<length:
                return s
        else:
                mid='..'+str(lens)+'..'
                half=int((length-len(mid))/2)
                return s[0:half]+mid+s[-half:]

def ReadBrackets(s,brackets,cur=0):
        '''Read the content inside the brackets, cur should be pointing to bracket[0]
s: string containing the brackets and contents
brackets: e.g. ['<ID>','</ID>']
cur: cursor position to begin with

rtype:(content,lastCursor)
content: content without the brackets
lastCursor: last cursor position at the end of bracket

returns '' and cur doesn't move if no valid brackets are found
raises ValueError if brackets in s are not paired
'''
        if s[cur]!=brackets[0][0]:
                return ('',cur)
        org=cur
        minl=s.find(brackets[0],cur+1)
        minr=s.find(brackets[1],cur+1)
        lenl=len(brackets[0])
        lenr=len(brackets[1])
        depth=1
        lens=len(s)
        while cur<lens:
                if NegComp(minr,minl):
                        cur=minr
                        minr=s.find(brackets[1],cur+1)
                        depth-=1
                else:
                        if minr==-1:
                                raise ValueError('brackets not paired in {'+s[cur:]+'}')
                        cur=minl
                        minl=s.find(brackets[0],cur+1)
                        depth+=1
                if depth==0:
                        break
        return s[org+lenl:cur],cur+lenr
def atoi(s):
        '''returns an int if s can be parsed to one, else just return s
'''
        try:
                return int(s)
        except Exception:
                return s
def Escape(s,escs):
        '''Returns the escaped str , escs is a (unescaped,escaped) list, like:
escs=[('"',r'\q'),('\\',r'\\'),("'",r'\p'),('/',r'\s'),('\n',r'\r\n')]
the order should be carefully designed to achieve the right replacement
'''
        r=s
        for t in escs:
                r=r.replace(t[0],t[1])
        return r
def Unescape(s,escs):
        '''The reverse of Escape, note that the replacement is in reverse order of escs
'''
        unesc=escs[:]
        unesc.reverse()
        r=s
        for t in unesc:
                r=r.replace(t[1],t[0])
        return r
        





__all__ = ['alib','waText','waFile','wa','wad','SvgScreen',
    'baseMod','loopMod','menu',
    'starMessager','wsServer','httpServer','wsCtrl']
#init alib
from .alib import alib
from .wadapter import Wa,SvgScreen
from . import wadapter as wad
from . import waText
from . import waFile
wa = Wa()

#init baseMod
from .starMod import baseMod,loopMod,menu

#init messager and servers
from .starServer import sm,wsServer,httpServer,wsCtrl
starMessager = sm




#基础模块类，作为各种功能模块的基础，推荐直接继承baseMod，然后重载show和InputKey的部分
#menuSelect属性用于记录进入这个模块的菜单索引，在backToMenu函数中调用
#show和InputKey可以重载为需要的功能
class baseMod():
    def __init__(self,menuInfo):
        self.menuInfo = menuInfo
        self.msgr = menuInfo['sm']
        self.dev = self.msgr.shipei.name

    def start(self):
        self.show()
    def backToMenu(self):
        self.msgr.workModule = menu(**self.menuInfo)
        self.msgr.workModule.show()
    def show(self):
        self.output("默认模块：\n该模块尚未开发，\n按任意键返回。")
    def InputKey(self,key):
        self.backToMenu()
    def output(self,str):
        self.msgr.output(str)

#基础的循环采集模式
#只需要重载loopUnit函数即可
class loopMod(baseMod):
    def __init__(self,mi):
        baseMod.__init__(self,mi)
        self.pause = False
        self.stop = False
        self.pauseWait = 0.5
        self.loopDelay = 0.5
    def loopUnit(self):
        self.output("Looping...")
    def loop(self,_=None):
        while not self.stop:
            while self.pause:
                pass
                time.sleep(self.pauseWait)
            self.loopUnit()
            import time
            time.sleep(self.loopDelay)
        self.backToMenu()
        self.msgr.workModule.show()
    def show(self):
        if self.dev != 'zkb':
            import threading
            t = threading.Thread(target=self.loop)
            t.start()
        else:
            import _thread as th
            th.start_new_thread(self.loop,(None,))
    def InputKey(self,key):
        if not key:
            return
        if key in "UAPHD":
            self.pause = not self.pause
        elif key in "LBNY":
            self.stop = True

#关机面板：内容为纯黑屏        
#暂时退出，关闭屏幕，按下任意键重新启动
class offStatus(baseMod):
    def __init__(self,mi):
        baseMod.__init__(self,mi)
    def show(self):
        self.output(" ")

class menu():
    def __init__(self,sm,initData,select=([],0),):
        self.data = initData
        self.submenu = select[0] #第一个数字代表哪个子目录，列表代表根目录；第二个为当前选中元素的下标
        self.pointer = select[1]
        self.msgr = sm
    def selectedSub(self):
        menuItem = self.data
        for i in self.submenu:
            keys = list(menuItem.keys())
            keys.sort()
            menuItem = menuItem[keys[i]]
        return menuItem        
    def show(self):
        menuItem = list(self.selectedSub().keys())
        menuItem.sort()
        output = ''
        if len(menuItem)<3:
            for _ in range(3-len(menuItem)):
                menuItem.append('\n')
        if self.pointer == 0:
            displayList = [0,1,2]
        elif self.pointer == len(menuItem)-1:
            displayList = [len(menuItem)-3,len(menuItem)-2,len(menuItem)-1]
        else:
            displayList = [self.pointer-1,self.pointer,self.pointer+1]
        for (i,item) in enumerate(menuItem):
            if i in displayList:
                output += "|* " + item + '\n' if(i == self.pointer) else "|  " + item + '\n'

        mtitle = ""
        if self.submenu:
            menuItem = self.data
            for i in self.submenu:
                mtitle = list(menuItem.keys())[i]
                menuItem = menuItem[list(menuItem.keys())[i]]
            mtitle += '\n'
        else:
            mtitle = '主菜单\n'
        if self.msgr.shipei.name == 'zkb':
            output += " A    ←    ↓    ↑    →    B"
        else:
            output = mtitle + output
        self.msgr.output(output)
    def subLength(self):
        return len(self.selectedSub())
    def movePointer(self,shift):
        self.pointer = (self.pointer + shift) % self.subLength()
    def enterSub(self):
        keys = list(self.selectedSub().keys())
        keys.sort()
        sSub = self.selectedSub()[keys[self.pointer]]
        if isinstance(sSub,dict):
            self.submenu.append(self.pointer)
            self.pointer = 0
            return False
        elif sSub == 0:
            self.exitSub()
        else:
            menuInfo = {"sm":self.msgr,"initData":self.data,"select":(self.submenu,self.pointer)}
            self.msgr.workModule = sSub(menuInfo)
            self.msgr.workModule.start()
            return True#返回值代表是否进入了新的Mod
    def exitSub(self):
        if len(self.submenu):
            self.pointer = self.submenu.pop()
    def InputKey(self,key):
        if not key:
            return
        self.msgr.msg=""
        if key in "UuH":
            self.movePointer(-1)
        elif key in "DdTAP":
            self.movePointer(1)
        elif key in "ROBN":
            if self.enterSub():
                return
        elif key in "LY":
            self.exitSub()
        self.show()


#! /usr/bin/env python3

"""
wadapter
=======================
Created by Cerulany 2022 <www.zhihu.com/people/cerulany>
    with lrm

Adapts numpy etc. commonly used python modules, so that same code can run on fullfledged python, as well as in a reduced python environment (e.g. micropython, qpython)

"""
# NOTE: the actual command documentation is collected from docstrings of the
# commands and is appended to __doc__ after the class has been defined.
"""
Revisions
=================
221202 by Spectre Lee
    把加法器的驱动转移到单独的文件中，
    该文件专注适配器。
    给每个适配类添加了name属性，可以直接调用name判断当前的设备类型。
221115 by Spectre Lee
    楠哥的继承写的有问题，
    class newClass(fClass):
        def __init__(self):
            fClass.__init__(self)
            pass
    必须这样写newClass才会继承fClass的init，否则是覆盖（重载）
221109 by Spectre Lee
    在SpXxx中添加了DrawStdLine.
    添加了Spwin的模式，一方面可以再win下调试，另一方面避免修改Shipei父类带来的bug。
    修改了checkKey的逻辑，现在会运行到检测到输入才停止，是一种等待操作。
221027 SpEsp32, 将SpZkb, SpWlb归为其子类
    复用代码，1030 lines
221018
    modified YsKey of Zkb, add timedelta display
    1041 lines
221012 by Spectre Lee
    Shipei类里添加self.i2c=none,不然win下不能运行
    为SvgScreen添加了DrawStdLines,将带有\n的几行数据格式化输出为四行的形式。
    对SvgScreen的位置和字号进行了比例缩放，适配不同尺寸的显示屏。
221009 改self.ai2c为self.i2c，增加通用性
    SvgScreen类,SpSmp和SpBdz增加SvgScreen
    1015 lines
220929 SpBdz fallback
    SIp fallback
220927 增加aenv.fPyL, LoopShowBme等
    Zkb上验证可用
220906 MCPioeb的程doのq283 加入debug选项
    修复Shipei.Sleep
    added 0x5a MLX90614 temperature sensor
    Zkb ad rwd but may have some problems
220905 button_a.value, button_b.value
    烫74283的程getのcheckLogic
    MCPioeb的程doのq283
    MCPioeb的程loopの74283speed 改为 LoopShow283
220902 repaired microPython I2C writeto_mem compatibility
    passed test on wlb
220831 repaired button a b simulation
    unified wlb DrawText, 626 lines
220829 add waFile, waText import as common fascade
    initDisplay for SpSmp
    add
220825 deal with i2c compatibility
    alib dict
    wa.LsScanI2c
220824 add SMbus, SpBdz
220823 增加Wa, SpZkb
    zkb demo成功了！
    Key Demo and CheckKey
    aI2c
220818 change sMac to shMac
    add SIp
    alib has some problems in microPython
220812 增加a支持的平台和a支持的设备,
    AWaEnv, Hello, Draw的框架，116 lines
220331 numpy, smbus for qpython

"""
try:
    from . import alib, waFile, waText
    from .alib import alib
except:
    from alib import alib,waFile,waText
aWaEnv={'sWal':'WalArtmin'}

a支持的平台='''[Python3|3.6+在电脑和GT0上运行的]
[QPython3|在安卓平台上运行的]
[microPython|在ESP32等微控制器上运行的]'''

a支持的设备='''[oDraw|[#|小屏幕控制]
[SSD1306|Pioneer600上的黄青屏等]
[mPython|掌控板上的黑白屏]
[Wlb|未来板上的彩屏]]

[I2C设备|
[LTR390|可见光、紫外传感器]
[mcp23017|IO扩展]
[MLX90614|红外传感]]

[组合设备|
[Pioneer600|Waveshare, 能插到GT0上的扩展板，包含SSD1306]
[UPS HAT|Waveshare, 不间断电源]
[GTmcp|包含mcp23017和LTR390的扩展板]
[GTamp|PKU RCCE, 可测转移曲线的扩展板]]'''
#以上模块可在GTixi的任何主机上使用

def AWaEnv():
    '''收集当前系统信息'''
    import os
    try:
        un=os.uname()
        aWaEnv['sSysname']=un.sysname
        aWaEnv['sNodename']=un.nodename
        aWaEnv['sRelease']=un.release
        aWaEnv['sVersion']=un.version
        aWaEnv['sMachine']=un.machine
    except AttributeError:
        #那估计是windows系统
        import platform
        un=platform.uname()
        aWaEnv['sNodename']=un.node
        aWaEnv['sSysname']=un.system
        aWaEnv['sRelease']=un.release
        aWaEnv['sVersion']=un.version
        aWaEnv['sMachine']=un.machine #architecture
        aWaEnv['sProcessor']=un.processor
    try:
        import uuid
        aWaEnv['iMac']=uuid.getnode()
        aWaEnv['shMac']=hex(aWaEnv['iMac'])[2:]
    except ImportError:
        try:
            import network
            #microPython here
            wlan=network.WLAN()
            bMac=wlan.config('mac')
            iMac=0
            for b in bMac:
                #print(b)
                iMac=iMac<<8
                iMac+=b
            aWaEnv['iMac']=iMac
            aWaEnv['shMac']=hex(aWaEnv['iMac'])[2:]
        except ImportError:
            #no network
            aWaEnv['iMac']=0
            aWaEnv['shMac']='none'
    except:
        pass
    return aWaEnv

def Hello():
    aWaEnv=AWaEnv()

aWaEnv['fPyL']=0
'''fPyL: Python环境评级
1.0 microPython 可用库：sys, os, random, math, time
2.0 bdz QPython 可用库：
3.0 Python36 可用库：numpy, PIL, scipy
3.5 smp Python：qcodes, graphviz, latex
'''
class SvgScreen(): #svg虚拟屏幕
    def __init__(self,size=(128,64)):
        self.tSize=size
        self.lContent=[]
        self.sSvg=''
    def Generate(self): #
        sSvg='<svg width="{}" height="{}" xmlns="http://www.w3.org/2000/svg" version="1.1" style="background:#000">\n'.format(self.tSize[0],self.tSize[1])
        sSvg+='\n'.join(self.lContent)
        sSvg+='\n</svg>'
        #print(sSvg)
        #sSvg.replace('\n','')
        return sSvg.replace('\n','') # 加上\n的话在传送门里显示会有点问题
    def DispShow(self):
        self.sSvg=self.Generate()
        return self.sSvg
    def DispClear(self):
        self.lContent=[]
    def DrawText(self,tPos,sTxt,c=None):
        if c is None:
            if tPos[1]<0.1:
                c="#ee0"
            else:
                c='#0ee'
        fontsize = self.tSize[1] * 3 // 16#起点不一样, svg是左下，zkb等是左上
        self.lContent.append('<text x="{}" y="{}" font-size="{}" font-family="monospace" style="fill:{}" xml:space="preserve">{}</text>'.format(int(self.tSize[0]*tPos[0]),int(self.tSize[1]*tPos[1])+fontsize,fontsize,c,sTxt))
    def DrawStdLines(self,input):
        self.DispClear()
        if input:
            lines = input.split("\n")
            for i in range(len(lines)):
                if i<4:
                    self.DrawText((0,0.04+0.2375*i),lines[i])
        return self.Generate()


class Shipei(): #统一Fascade
    def __init__(self,aenv):
        self.aenv=aenv # aWaEnv
        self.i2c = None
        self.name = "default"

    def DispChar(self,txt):
        if isinstance(txt,str):
            lines=txt.split('\n')
        else:
            lines=txt
        self.DispClear()
        try:
            self.DrawText((0,0),lines[0],c=(200,200,0))
            self.DrawText((0,15),lines[1],c=(0,200,200))
            self.DrawText((0,30),lines[2],c=(0,200,200))
            self.DrawText((0,45),lines[3],c=(0,200,200))
        except IndexError:
            pass
        self.DispShow()
    def YsDisp(self): #demonstrate display function
        txt=str(self.aenv)
        self.LoopText(txt)
        self.DispClear()

    def Sleep(self,fDurS):
        import time
        time.sleep(fDurS)
    def YsKey(self):
        import time
        iStart=time.time()
        while True:
            try:
                self.DispChar('KeyDemo  {}\n CurKey: '.format(waText.STimeDiffS(time.time()-iStart))+self.cKey)
                self.Sleep(.5)
            except KeyboardInterrupt:
                import _thread
                _thread.exit()
    def LoopText(self,sText,sTitle='文本查看'):

        lines=sText.splitlines()
        nl=len(lines)
        i=0
        k=self.CheckKey()
        while k!='L':
            if i==0:
                l1=''
            else:
                l1=lines[i-1]
            l2=lines[i]
            if i==nl-1:
                l3=''
            else:
                l3=lines[i+1]
            self.DispChar([sTitle,l1,l2,l3])

            k=self.CheckKey()

            if k=='U':
                i-=1
            elif k=='D':
                i+=1

            if i>nl-1:
                i=nl-1
            elif i<0:
                i=0
            self.Sleep(.2)
        #self.DispClear()
        self.DispShow()
    def LsScanI2c(self):
        #default scan, rp seems does not have i2c scan function
        i2c=self.oi2c
        lsI2c=[]
        for h in range(255):
            try:
                r=i2c.read_byte(h)
                print('Found I2C 0x{:x}({}): {:0x}'.format(h,h,r))
                lsI2c.append('0x{:x}'.format(h))
            except OSError:
                pass
        print('Done scan I2C')
        return lsI2c
class SpEsp32(Shipei): #Esp32, includes Zkb, Wlb etc.
    def ASetupI2c(self):
        # 使用self.oi2c
        def wbd(address,pos,dat,i2c=self.oi2c):
            return i2c.writeto_mem(address,pos,chr(dat).encode())
        def rbd(address,pos,i2c=self.oi2c):
            return ord(i2c.readfrom_mem(address,pos,1))
        def rwd(address,pos,i2c=self.oi2c):
            r=i2c.readfrom_mem(address,pos,2)
            print(r)
            return ord(r[0])<<8+ord(r[1]) #maybe problems here
        def ribd(address,pos,iLen,i2c=self.oi2c):
            r=i2c.readfrom_mem(address,pos,iLen)
            return r
        class AI2C():
            def __init__(self,write_byte_data,read_byte_data,read_word_data,read_i2c_block_data):
                self.read_word_data = read_word_data
                self.write_byte_data = write_byte_data
                self.read_byte_data = read_byte_data
                self.read_i2c_block_data = read_i2c_block_data
        return AI2C(write_byte_data=wbd, read_byte_data=rbd, read_word_data=rwd,read_i2c_block_data=ribd)
        return alib(dict(write_byte_data=wbd, read_byte_data=rbd,read_word_data=rwd,
                            read_i2c_block_data=ribd))
    def CheckKey(self):
        return self.cKey
    def LsScanI2c(self):
        liI2c=self.oi2c.scan()
        lsI2c=['0x{:x}'.format(h) for h in liI2c]
        return lsI2c
    def YsKey(self): #demo key
        from wadapter import waText
        import time
        iStart=time.time()
        while True:
            try:
                self.DispChar('KeyDemo  {}\nCurKey: {}\n    iLastA: {}\n    iLastB: {}'.format(waText.STimeDiffS(time.time()-iStart),self.cKey,self.iLastA,self.iLastB))
                self.Sleep(.5)
            except KeyboardInterrupt:
                import _thread
                _thread.exit()

class SpZkb(SpEsp32): #掌控板适配
    def __init__(self,aenv):
        #SpEsp32.__init__(self,aenv)
        self.name="zkb"
        self.aenv=aenv
        from mpython import oled
        self.DispShow=oled.show
        from mpython import i2c
        self.oi2c=i2c
        self.i2c=self.ASetupI2c()
        self.DrawStdLine = self.DispChar
        self.oSvg = SvgScreen()
    def CheckKey(self):
        from mpython import sleep_ms,button_a,button_b,touchPad_P,touchPad_Y,touchPad_T,touchPad_H,touchPad_O,touchPad_N
        while True:
            sleep_ms(100)           

    def DrawText(self,tPos,sTxt,c=None):
        from mpython import oled
        oled.DispChar(sTxt,tPos[0],tPos[1])
    def DispClear(self):
        from mpython import oled
        oled.fill(0)
        #oled.show()
    def Sleep(self,fDurS):
        from mpython import sleep_ms
        sleep_ms(int(fDurS*1000))
    def DrawStdLines(self,input):
        from mpython import oled
        oled.fill(0)
        if input:
            lines = input.split("\n")
            for i in range(len(lines)):
                if i<4:
                    oled.DispChar(lines[i],0,1+15*i)
            oled.show()

class SpSmp(Shipei): #树莓派适配
    def __init__(self,aenv):
        Shipei.__init__(self,aenv)
        self.name="smp"
        self.aenv=aenv
        self.cSoftKey=''
        import math
        import time

        import spidev as SPI
        import SSD1306

        import smbus
        self.oi2c=smbus.SMBus(1)
        i2c=self.oi2c
        class AI2C():
            def __init__(self,write_byte_data,read_byte_data,read_i2c_block_data):
                self.write_byte_data = write_byte_data
                self.read_byte_data = read_byte_data
                self.read_i2c_block_data = read_i2c_block_data
        self.i2c=AI2C(write_byte_data=i2c.write_byte_data, read_byte_data=i2c.read_byte_data, read_i2c_block_data=i2c.read_i2c_block_data)
        self.hasSsd=None
        self.oSvg=None
        self.InitDisplay()

    def InitDisplay(self):
        try:
            import spidev as SPI
            import SSD1306
            from PIL import Image
            from PIL import ImageFont
            from PIL import ImageDraw

            # Raspberry Pi pin configuration:
            RST = 19
            DC = 16
            bus = 0
            device = 0

            # 128x32 display with hardware SPI:
            disp = SSD1306.SSD1306(rst=RST,dc=DC,spi=SPI.SpiDev(bus,device))

            # Initialize library.
            disp.begin()

            # Get display width and height.
            width = disp.width
            height = disp.height
            self.width=width
            self.height=height

            # Clear display.
            disp.clear()
            disp.display()

            # Create image buffer.
            # Make sure to create image with mode '1' for 1-bit color.
            self.image = Image.new('1', (width, height))

            # Get drawing object to draw on image.
            self.draw = ImageDraw.Draw(self.image)
            self.disp=disp
            # Draw a black filled box to clear the image.
            #draw.rectangle((0,0,width,height), outline=0, fill=0)

            fontpath='simsun.ttf'
            self.font=ImageFont.truetype(fontpath,15)
            self.hasSsd=True
        except Exception as e:
            print('SSD初始化出错:',e)
            self.hasSsd=False
        self.oSvg=SvgScreen()

    def CheckKey(self):
        import RPi.GPIO as GPIO
        import smbus
        import time
        address = 0x20
        bus = smbus.SMBus(1)
        ans=''
        while True:
            time.sleep(0.15)
            bus.write_byte(address,0x0F|bus.read_byte(address))
            value = bus.read_byte(address) | 0xF0
            if value != 0xFF:
                if (value | 0xFE) != 0xFF:
                    ans='L'
                elif (value | 0xFD) != 0xFF:
                    ans='U'
                elif (value | 0xFB) != 0xFF:
                    ans='D'
                else :
                    ans='R'
                return ans
            elif self.cSoftKey:
                ans=self.cSoftKey
                self.cSoftKey=''
                return ans   
    def DrawText(self,tPos,sTxt,c=None):
        if self.hasSsd:
            self.draw.text(tPos,sTxt,font=self.font,fill=1)
        self.oSvg.DrawText(tPos,sTxt)
    def DispClear(self):
        if self.hasSsd:
            self.draw.rectangle((0,0,self.width,self.height), outline=0, fill=0)
        self.oSvg.DispClear()
    def DispShow(self):
        if self.hasSsd:
            self.disp.image(self.image)
            self.disp.display()
        return self.oSvg.DispShow()
    
    def DrawStdLines(self,input):
        if self.hasSsd:
            self.draw.rectangle((0,0,self.width,self.height), outline=0, fill=0)
        else:
            return
        if input:
            lines = input.split("\n")
            for i in range(len(lines)):
                if i<4:
                    if self.hasSsd:
                        self.draw.text((0,1+15*i),lines[i],font=self.font,fill=1)
            self.disp.image(self.image)
            self.disp.display()

    def Sleep(self,fDur):
        import time
        time.sleep(fDur)

class SpBdz(Shipei): #GT1 adapter
    def __init__(self,aenv):
        Shipei.__init__(self,aenv)
        self.name="bdz"
        self.aenv=aenv
        self.cSoftKey=''
        try:
            from smbus2 import SMBus
            self.oi2c=SMBus(6)
            i2c=self.oi2c
            class AI2C():
                def __init__(self,write_byte_data,read_byte_data):
                    self.write_byte_data = write_byte_data
                    self.read_byte_data = read_byte_data
            self.i2c=AI2C(write_byte_data=i2c.write_byte_data, read_byte_data=i2c.read_byte_data)
        except:
            print('SMbus2 not available, install it to use it')
            self.oi2c=None
            self.i2c=None
        self.oSvg=SvgScreen()
    def CheckKey(self):
        while True:
            import time
            time.sleep(100) #by Spectre Lee, 预留给bdz实体键可以使用时
        ans = None
        if self.cSoftKey:
            ans=self.cSoftKey
            self.cSoftKey=''
        return ans
    def DrawText(self,tPos,sTxt,c=None):
        self.oSvg.DrawText(tPos,sTxt,c)
    def DispClear(self):
        self.oSvg.DispClear()
    def DispShow(self):
        return self.oSvg.DispShow()
    def DrawStdLines(self,input):
        return self.oSvg.DrawStdLines(input)

class SpWlb(SpEsp32):
    def __init__(self,aenv):
        SpEsp32.__init__(self,aenv)
        self.name="wlb"
        self.aenv=aenv

        self.iLastA=0
        self.iLastB=0
        self.cKey=''
        self.needClear=True

        from future import i2c
        self.oi2c=i2c
        self.i2c=self.ASetupI2c()

        import _thread
        def ThreadBtnScan(_,wa=self):
            #loop scan AB虚拟六个键的功能：LRUDOX
            tButA=False
            tButB=False #last button state
            wa.iLastA=100
            wa.iLastB=100 #rounds since last button up
            from future import sensor
            while True:
                try:
                    if sensor.btnValue('a'): #button press down
                        tButA=True
                        if wa.cKey=='':
                            if wa.iLastA<3:
                                wa.cKey='X'
                            elif wa.iLastB<3:
                                wa.cKey='L'
                            else:
                                wa.cKey='U'
                    else:
                        if tButA:
                            #if released, record last up
                            wa.iLastA=0
                        tButA=False
                    if sensor.btnValue('b'):
                        tButB=True
                        if wa.cKey=='':
                            if wa.iLastB<3:
                                wa.cKey='O'
                            elif wa.iLastA<3:
                                wa.cKey='R'
                            else:
                                wa.cKey='D'
                    else:
                        if tButB:
                            wa.iLastB=0
                        tButB=False
                    wa.iLastA = wa.iLastA+1 if wa.iLastA<100 else 100
                    wa.iLastB = wa.iLastB+1 if wa.iLastB<100 else 100
                    if (not tButA) and (not tButB):
                        self.cKey=''
                    wa.Sleep(.05)
                except OSError as e:
                    print(e)
        _thread.start_new_thread(ThreadBtnScan,(1,))

        from future import sleep
        self.Sleep=sleep

    def DrawText(self,tPos,sTxt,c=(255,255,255)):
        from future import screen
        ipos=0
        iLenTxt=len(sTxt)
        IsCh=lambda x:ord(x)>128
        isCh=IsCh(sTxt[ipos])
        iPosX=tPos[0]+16
        iPosY=tPos[1]+32
        while ipos<iLenTxt:
            iLen=0
            while ipos+iLen<iLenTxt and isCh==IsCh(sTxt[ipos+iLen]):
                iLen+=1
            if isCh:
                screen.textCh(sTxt[ipos:ipos+iLen],iPosX,iPosY,1,c)
                iPosX+=iLen*16
            else:
                screen.text(sTxt[ipos:ipos+iLen],iPosX,iPosY,1,c)
                iPosX+=iLen*8
            ipos+=iLen
            isCh=not isCh
    def DispClear(self):
        from future import screen
        #screen.fill(0)
        screen.clear()
    def DispShow(self):
        from future import screen
        screen.refresh()
    def DrawStdLines(self,input):
        return self.oSvg.DrawStdLines(input)

class SpWin(Shipei):
    def __init__(self,aenv):
        Shipei.__init__(self,aenv)
        self.name = "win"
        self.oSvg = SvgScreen()
    def CheckKey(self):
        while True:
            import time
            time.sleep(100) #by Spectre Lee, 
    def DrawText(self,tPos,sTxt,c=None):
        self.oSvg.DrawText(tPos,sTxt,c)
    def DispClear(self):
        self.oSvg.DispClear()
    def DispShow(self):
        return self.oSvg.DispShow()
    def DrawStdLines(self,input):
        return self.oSvg.DrawStdLines(input)

def SIp():
    #get Ip address
    import socket
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    except:
        ip='#no network connection'
    finally:
        s.close()
    aWaEnv['sIp']=ip
    return ip
def Wa(aenv=aWaEnv):
    if aenv == 'debug':
        return SpSmp(aenv)
    elif 'mpython' in aenv['sMachine']:
        return SpZkb(aenv)
    elif 'Future' in aenv['sMachine']:
        return SpWlb(aenv)
    elif 'v7l' in aenv['sMachine']:
        if 'localhost' in aenv['sMachine']:
            return SpBdz(aenv)
        else:
            return SpSmp(aenv)
    else:
        #print(aenv.sMachine)
        return SpWin(aenv)

Hello()
wa=Wa()
print(wa.name)
##################################################
DEBUG=False
aI2c='''[#comment|此体系中器件有统一i2c地址]
[0x20|PCF8574IO扩展（Pioneer600）]
[0x26|GTmcp的mcp23017]
[0x39|APDS9960光线颜色传感器]
[0x42|UPS HAT]
[0x48|PCF8591ADDA（Pioneer600）]
[0x53|LTR390紫外光光感]
[0x5a|MLX90614红外传感]
[0x68|DS3231时钟模块（Pioneer600）]
[0x76|BMP280温度压力传感（Pioneer600）]
[0x77|BME280温湿度压力传感器]'''
strWaEnv = str(aWaEnv).replace("{","[").replace("}","]").replace(": ","|").replace("'","").replace(", ","][")
print(strWaEnv)

if __name__ == "__main__":
    pass
